@@ -3,6 +3,6 @@ from django.utils.translation import gettext_lazy as _
 
 
 class SalesmanAdminApp(AppConfig):
-    name = 'salesman.admin'
-    label = 'salesmanadmin'
+    name = "salesman.admin"
+    label = "salesmanadmin"
     verbose_name = _("Salesman Admin")
