@@ -1,7 +1,14 @@
+from __future__ import annotations
+
+from typing import Any
+
 from django.core.exceptions import ValidationError  # noqa
 
 
-def validate_basket_item(attrs: dict, context: dict = {}) -> dict:
+def validate_basket_item(
+    attrs: dict[str, Any],
+    context: dict[str, Any] = {},
+) -> dict[str, Any]:
     """
     Default basket item validator function. Can be overrider by providing
     a path to a function in ``SALESMAN_BASKET_ITEM_VALIDATOR`` setting.
@@ -19,7 +26,10 @@ def validate_basket_item(attrs: dict, context: dict = {}) -> dict:
     return attrs
 
 
-def validate_extra(value: dict, context: dict = {}) -> dict:
+def validate_extra(
+    value: dict[str, Any],
+    context: dict[str, Any] = {},
+) -> dict[str, Any]:
     """
     Default extra validator function. Can be overriden by providing a
     dotted path to a function in ``SALESMAN_EXTRA_VALIDATOR`` setting.
