@@ -3,6 +3,6 @@ from django.utils.translation import gettext_lazy as _
 
 
 class SalesmanCheckoutApp(AppConfig):
-    name = 'salesman.checkout'
-    label = 'salesmancheckout'
+    name = "salesman.checkout"
+    label = "salesmancheckout"
     verbose_name = _("Salesman Checkout")
