@@ -18,7 +18,7 @@ def generate_ref(request: HttpRequest) -> str:
         str: New order reference
     """
     year = timezone.now().year
-    Order = get_salesman_model('Order')
+    Order = get_salesman_model("Order")
     last = Order.objects.filter(date_created__year=year, ref__isnull=False).first()
-    increment = int(last.ref.split('-')[1]) + 1 if last and last.ref else 1
-    return f'{year}-{increment:05d}'
+    increment = int(last.ref.split("-")[1]) + 1 if last and last.ref else 1
+    return f"{year}-{increment:05d}"
