@@ -6,8 +6,8 @@ from salesman.checkout.views import CheckoutViewSet
 from salesman.orders.views import OrderViewSet
 
 router = DefaultRouter()
-router.register('basket', BasketViewSet, basename='salesman-basket')
-router.register('checkout', CheckoutViewSet, basename='salesman-checkout')
-router.register('orders', OrderViewSet, basename='salesman-order')
+router.register("basket", BasketViewSet, basename="salesman-basket")
+router.register("checkout", CheckoutViewSet, basename="salesman-checkout")
+router.register("orders", OrderViewSet, basename="salesman-order")
 
 urlpatterns = router.urls + payment_methods_pool.get_urls()
